#include <stdio.h>
#include <string.h>	   
#include "kernel.h"

#define UART_PORTNO 2

const char AppVersion[] = "USR V0.01 #";

char dbg_msg[128];

int uart_init(void);
void close_cnn(void);

static int lSocket, cnn_flag;

void user_main(void *pvParameters)
{				  	
	struct sockaddr_in dest;
    int nbytes, mode;
    char buffer[128];

	cnn_flag = -1;

	sprintf(dbg_msg, "E06 %s(%s %s)\r\n", AppVersion, __DATE__, __TIME__);
    console_write(dbg_msg);

	if (uart_init() < 0) {
		return;
	}
	while(1) {
		if (is_network_ready() == NETWORK_STATUS_READY) {
			break;
		}
		vTaskDelay(10);
	}
	sprintf(dbg_msg, "%s\r\n", "network ready.");
    console_write(dbg_msg);

	lSocket = socket(AF_INET, SOCK_STREAM, 0);
	if (lSocket < 0) {
		sprintf(dbg_msg, "%s\r\n", "socket failed");
    	console_write(dbg_msg);
		return;
	}

	memset((char *)&dest, 0, sizeof(dest));
	dest.sin_family = AF_INET;
	dest.sin_port = htons(4002);
	inet_aton("192.168.2.37", &dest.sin_addr.s_addr);

	while (1) {	 

		if (cnn_flag != 0) {
			cnn_flag = connect(lSocket, (struct sockaddr*)&dest, sizeof(dest));
			if (cnn_flag != 0) {
				close_cnn();
				vTaskDelay(10);
				continue;
			} else {
				sprintf(dbg_msg, "%s\r\n", "connected");
				console_write(dbg_msg);	
				mode = 1;
				ioctl(lSocket, FIONBIO, &mode);	
			}
		}
       	nbytes = recv(lSocket, buffer, sizeof(buffer),0);
        if (nbytes > 0) {
			sprintf(dbg_msg, "recv:%d\r\n", nbytes);
			console_write(dbg_msg);
			prg_uart_puts(UART_PORTNO, buffer, nbytes);
		} else if (nbytes < 0) {
			if (errno != EWOULDBLOCK) {
				sprintf(dbg_msg, "%s\r\n", "disconnect");
				console_write(dbg_msg);	 
				close_cnn();
				continue;
			}
		}
		nbytes = prg_uart_gets(UART_PORTNO, buffer, sizeof(buffer));
		if (nbytes > 0) {
			sprintf(dbg_msg, "uart recv:%d\r\n", nbytes);
			console_write(dbg_msg);
			send(lSocket, buffer, nbytes, 0);
		}

		vTaskDelay(1);

    }

}		  

int uart_init(void)
{
	if (prg_uart_Open(UART_PORTNO, RS232) < 0) {
		sprintf(dbg_msg, "%s\r\n", "uart open failed");
    	console_write(dbg_msg);	
		return -1;
	}
	if (prg_uart_Config(UART_PORTNO, B19200, "N81") < 0) {
		sprintf(dbg_msg, "%s\r\n", "uart Config failed");
    	console_write(dbg_msg);	
		prg_uart_Close(UART_PORTNO);
		return -1;
	}
	//no flow control
	if (prg_uart_FlowControl(UART_PORTNO, "NA") < 0) {
		sprintf(dbg_msg, "%s\r\n", "uart Config failed");
    	console_write(dbg_msg);	
		prg_uart_Close(1);
		return -1;
	}
	//interval timeout: vtime*0.10seconds(655 most), min char: vmin(512 bytes most)
	//vtime=0,vmin=0:return is immediate				 
	if (pgr_uart_RxTimeoutThreshold(UART_PORTNO,0,0) < 0) {
		sprintf(dbg_msg, "%s\r\n", "uart RxTimeout failed");
    	console_write(dbg_msg);	
		prg_uart_Close(1);
		return -1;
	}
	return 0;
}

void close_cnn(void)
{
	closesocket(lSocket);
	lSocket = socket(AF_INET, SOCK_STREAM, 0);
	if (lSocket < 0) {
		sprintf(dbg_msg, "%s\r\n", "socket failed");
    	console_write(dbg_msg);
	}
	cnn_flag = -1;
}




